@@ -6,7 +6,7 @@ use crate::{
         font::{Font, FontImpl},
         Galley, LayoutJob,
     },
-    Texture, TextureAtlas,
+    FontImage, TextureAtlas,
 };
 
 // TODO: rename
@@ -52,13 +52,43 @@ pub enum FontFamily {
     Proportional,
 }
 
-/// The data of a `.ttf` or `.otf` file.
-pub type FontData = std::borrow::Cow<'static, [u8]>;
+/// A `.ttf` or `.otf` file and a font face index.
+#[derive(Clone, Debug, PartialEq)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
+pub struct FontData {
+    /// The data of a `.ttf` or `.otf` file.
+    pub font: std::borrow::Cow<'static, [u8]>,
+    /// Which font face in the file to use.
+    /// When in doubt, use `0`.
+    pub index: u32,
+}
+
+impl FontData {
+    pub fn from_static(font: &'static [u8]) -> Self {
+        Self {
+            font: std::borrow::Cow::Borrowed(font),
+            index: 0,
+        }
+    }
+
+    pub fn from_owned(font: Vec<u8>) -> Self {
+        Self {
+            font: std::borrow::Cow::Owned(font),
+            index: 0,
+        }
+    }
+}
 
 fn ab_glyph_font_from_font_data(name: &str, data: &FontData) -> ab_glyph::FontArc {
-    match data {
-        std::borrow::Cow::Borrowed(bytes) => ab_glyph::FontArc::try_from_slice(bytes),
-        std::borrow::Cow::Owned(bytes) => ab_glyph::FontArc::try_from_vec(bytes.clone()),
+    match &data.font {
+        std::borrow::Cow::Borrowed(bytes) => {
+            ab_glyph::FontRef::try_from_slice_and_index(bytes, data.index)
+                .map(ab_glyph::FontArc::from)
+        }
+        std::borrow::Cow::Owned(bytes) => {
+            ab_glyph::FontVec::try_from_vec_and_index(bytes.clone(), data.index)
+                .map(ab_glyph::FontArc::from)
+        }
     }
     .unwrap_or_else(|err| panic!("Error parsing {:?} TTF/OTF font file: {}", name, err))
 }
@@ -85,7 +115,7 @@ fn ab_glyph_font_from_font_data(name: &str, data: &FontData) -> ab_glyph::FontAr
 ///
 /// You can also install your own custom fonts:
 /// ```
-/// # use {epaint::text::{FontDefinitions, TextStyle, FontFamily}};
+/// # use {epaint::text::{FontDefinitions, TextStyle, FontFamily, FontData}};
 /// # struct FakeEguiCtx {};
 /// # impl FakeEguiCtx { fn set_fonts(&self, _: FontDefinitions) {} }
 /// # let ctx = FakeEguiCtx {};
@@ -93,7 +123,7 @@ fn ab_glyph_font_from_font_data(name: &str, data: &FontData) -> ab_glyph::FontAr
 ///
 /// // Install my own font (maybe supporting non-latin characters):
 /// fonts.font_data.insert("my_font".to_owned(),
-///    std::borrow::Cow::Borrowed(include_bytes!("../../fonts/Ubuntu-Light.ttf"))); // .ttf and .otf supported
+///    FontData::from_static(include_bytes!("../../fonts/Ubuntu-Light.ttf"))); // .ttf and .otf supported
 ///
 /// // Put my font first (highest priority):
 /// fonts.fonts_for_family.get_mut(&FontFamily::Proportional).unwrap()
@@ -110,10 +140,8 @@ fn ab_glyph_font_from_font_data(name: &str, data: &FontData) -> ab_glyph::FontAr
 #[cfg_attr(feature = "serde", serde(default))]
 pub struct FontDefinitions {
     /// List of font names and their definitions.
-    /// The definition must be the contents of either a `.ttf` or `.otf` font file.
     ///
-    /// `epaint` has built-in-default for these,
-    /// but you can override them if you like.
+    /// `epaint` has built-in-default for these, but you can override them if you like.
     pub font_data: BTreeMap<String, FontData>,
 
     /// Which fonts (names) to use for each [`FontFamily`].
@@ -139,22 +167,22 @@ impl Default for FontDefinitions {
         {
             font_data.insert(
                 "Hack".to_owned(),
-                std::borrow::Cow::Borrowed(include_bytes!("../../fonts/Hack-Regular.ttf")),
+                FontData::from_static(include_bytes!("../../fonts/Hack-Regular.ttf")),
             );
             font_data.insert(
                 "Ubuntu-Light".to_owned(),
-                std::borrow::Cow::Borrowed(include_bytes!("../../fonts/Ubuntu-Light.ttf")),
+                FontData::from_static(include_bytes!("../../fonts/Ubuntu-Light.ttf")),
             );
 
             // Some good looking emojis. Use as first priority:
             font_data.insert(
                 "NotoEmoji-Regular".to_owned(),
-                std::borrow::Cow::Borrowed(include_bytes!("../../fonts/NotoEmoji-Regular.ttf")),
+                FontData::from_static(include_bytes!("../../fonts/NotoEmoji-Regular.ttf")),
             );
             // Bigger emojis, and more. <http://jslegers.github.io/emoji-icon-font/>:
             font_data.insert(
                 "emoji-icon-font".to_owned(),
-                std::borrow::Cow::Borrowed(include_bytes!("../../fonts/emoji-icon-font.ttf")),
+                FontData::from_static(include_bytes!("../../fonts/emoji-icon-font.ttf")),
             );
 
             fonts_for_family.insert(
@@ -205,9 +233,10 @@ pub struct Fonts {
     definitions: FontDefinitions,
     fonts: BTreeMap<TextStyle, Font>,
     atlas: Arc<Mutex<TextureAtlas>>,
-    /// Copy of the texture in the texture atlas.
+
+    /// Copy of the font image in the texture atlas.
     /// This is so we can return a reference to it (the texture atlas is behind a lock).
-    buffered_texture: Mutex<Arc<Texture>>,
+    buffered_font_image: Mutex<Arc<FontImage>>,
 
     galley_cache: Mutex<GalleyCache>,
 }
@@ -230,7 +259,7 @@ impl Fonts {
             // Make the top left pixel fully white:
             let pos = atlas.allocate((1, 1));
             assert_eq!(pos, (0, 0));
-            atlas.texture_mut()[pos] = 255;
+            atlas.image_mut()[pos] = 255;
         }
 
         let atlas = Arc::new(Mutex::new(atlas));
@@ -256,7 +285,7 @@ impl Fonts {
 
         {
             let mut atlas = atlas.lock();
-            let texture = atlas.texture_mut();
+            let texture = atlas.image_mut();
             // Make sure we seed the texture version with something unique based on the default characters:
             texture.version = crate::util::hash(&texture.pixels);
         }
@@ -266,16 +295,11 @@ impl Fonts {
             definitions,
             fonts,
             atlas,
-            buffered_texture: Default::default(), //atlas.lock().texture().clone();
+            buffered_font_image: Default::default(), //atlas.lock().texture().clone();
             galley_cache: Default::default(),
         }
     }
 
-    #[deprecated = "Renamed to Fonts::new"]
-    pub fn from_definitions(pixels_per_point: f32, definitions: FontDefinitions) -> Self {
-        Self::new(pixels_per_point, definitions)
-    }
-
     #[inline(always)]
     pub fn pixels_per_point(&self) -> f32 {
         self.pixels_per_point
@@ -296,11 +320,11 @@ impl Fonts {
     }
 
     /// Call each frame to get the latest available font texture data.
-    pub fn texture(&self) -> Arc<Texture> {
+    pub fn font_image(&self) -> Arc<FontImage> {
         let atlas = self.atlas.lock();
-        let mut buffered_texture = self.buffered_texture.lock();
-        if buffered_texture.version != atlas.texture().version {
-            *buffered_texture = Arc::new(atlas.texture().clone());
+        let mut buffered_texture = self.buffered_font_image.lock();
+        if buffered_texture.version != atlas.image().version {
+            *buffered_texture = Arc::new(atlas.image().clone());
         }
 
         buffered_texture.clone()
